import {createProgramFromSource, ProgramError, ShaderError} from "./gl_util";

/// <reference path="./glsl.d.ts" />
import vert_src from "./vertex.vert";
import frag_src from "./fragment.frag";

import { RenderProgram } from "./program";
import * as G from "./glWrapper";
import { getUniformDefaultValue, UniformSet } from "./uniform";

export class Renderer2D implements RenderProgram{
    gl : WebGL2RenderingContext;
    uniforms : UniformSet;
    program: WebGLProgram;
    
    vao: WebGLVertexArrayObject;
    positionBuffer: G.VertexBuffer;
    indexBuffer: G.IndexBuffer;
    constructor(gl: WebGL2RenderingContext){
        this.gl = gl;
        this.uniforms = getUniformDefaultValue();
        try{
            this.program = createProgramFromSource(gl,vert_src,frag_src);
            gl.useProgram(this.program);
        }
        catch(e){
            if(e instanceof ShaderError){
                console.log(e.message,"\n",e.info);
            }
            else if(e instanceof ProgramError){
                console.log(e.message,"\n",e.info);
            }
            else throw e;
        }
    }
    prepare(){
        const gl = this.gl;
        const position = [
            -0.5,-0.5,
            0.5,-0.5,
            0.5,0.5,
            -0.5,0.5
        ];
        const index = [
            0,1,2,
            2,3,0
        ];
        this.vao = gl.createVertexArray();
        const posLoc = gl.getAttribLocation(this.program,"pos");
        gl.bindVertexArray(this.vao);
        
        this.positionBuffer = G.createVertexBuffer(gl,position);
        this.positionBuffer.bind(gl);
        gl.enableVertexAttribArray(posLoc);
        gl.vertexAttribPointer(posLoc,2,gl.FLOAT,false,0,0);

        this.indexBuffer = G.createIndexBuffer(gl,index);
        this.indexBuffer.bind(gl);
        this.settingUniform(this.uniforms);
    }
    useProgram(){
        this.gl.useProgram(this.program);
    }
    settingUniform(u:UniformSet){
        const gl = this.gl;
        this.uniforms = u;
        this.useProgram();
        const location = gl.getUniformLocation(this.program, "u_color"); //u_color 변수 위치를 참조
        gl.uniform4f(location, this.uniforms.redcolor, 0.3, 0.8, 1.0); //해당 위치에 0.2, 0.3, 0.8, 1.0 데이터를 전달
    }
    draw(){
        const gl = this.gl;
        gl.clearColor(0,0,0,0);
        gl.clear(gl.COLOR_BUFFER_BIT);

        gl.viewport(0,0,gl.canvas.width,gl.canvas.height);
        this.useProgram();
        gl.bindVertexArray(this.vao);
        this.indexBuffer.bind(gl);
        gl.drawElements(gl.TRIANGLES,this.indexBuffer.count,gl.UNSIGNED_SHORT,0);
    }
}